//
//  EstencioVehicle.swift
//  Concot
//
//  Created by Andreu Font Bardolet on 17/5/15.
//  Copyright (c) 2015 Andreu Font Bardolet. All rights reserved.
//

/*
Aquest fitxer amplia l'entitat de la BDD vehicles afegint funcions i propietats calculades que
permeten obtenir dades estadístiques del ús del vehicle, i realitzar operacións sobre el vehicle

*/

import Foundation

extension Vehicle {
    
    var contaquilometres: Double {
        //Retorna els quilòmetres que ha recorregut el vehicle
        get{
            return Double(self.obtenirKm())
        }
    }
    
    func distanciaDiariaRecorreguda() -> Double {
        //Calcula la mitjana de quilòmetres que es recorre cada dia
        let emplenaments = self.obtenirEmplenaments(false, max: 3, ple: false)
        if emplenaments.count > 1 {
            var distanciaAcumulada = 0.0
            let mesRecent = emplenaments.first?.data
            let diesDiferencia = mesRecent?.daysFrom(emplenaments.last!.data)
            var distanciaDarrerEmplenament = emplenaments.first!.contaquilometres
            for emplenament in emplenaments {
                distanciaAcumulada += distanciaDarrerEmplenament - emplenament.contaquilometres
                distanciaDarrerEmplenament = emplenament.contaquilometres
            }
            return distanciaAcumulada / Double(diesDiferencia!)
        }
        return 0
    }
    
    func previsioDeCostDiari() -> 	Double {
        //Calcula el cost diari segons la distància mitja diària recorreguda i els costs diaris fixes
        
        let costPrevistCarburant = self.previsioDeCostCarburantDiari()
        let costPrevistDesgast = self.previsioDeCostDesgastDiari()
        let costDesplacamentPrevist = costPrevistCarburant + costPrevistDesgast
        
        let costSeguroDiari = self.costDiariSeguro()
        let costImpostDiari = self.costDiariImpost()
        let costDiariVehicle = costSeguroDiari + costImpostDiari
        
        return costDesplacamentPrevist + costDiariVehicle
    }
    
    func previsioDeCostCarburantDiari() -> Double {
        //Calcula el consum en moneda diari de carburant segons la mitja de distància diaria recorreguda
        let distanciaDiariaPrevista = self.distanciaDiariaRecorreguda()
        let costDistanciaPrevist = self.obtenirCostCarburantPerUnitatDistancia() ?? 0.0
        
        return distanciaDiariaPrevista * costDistanciaPrevist
    }
    
    func previsioDeCostDesgastDiari() -> Double {
        //Calcula el cost diari de desgast de peces segons la distància diaria recorregida
        let distanciaDiariaPrevista = self.distanciaDiariaRecorreguda()
        let costDesgastPrevist = self.obtenirCostDesgastPerUnitatDistancia() ?? 0.0
        
        return distanciaDiariaPrevista * costDesgastPrevist
    }
    
    func totalCarburantPosat() -> Double {
        //Retorna el total de lítres introduits des de l'inici
        let emplenaments = self.obtenirEmplenaments(false)
        
        var quantitat = 0.0
        for emplenament in emplenaments {
            quantitat += emplenament.quantitat
        }
        return quantitat
    }
    
    func totalCarburantPagat() -> Double {
        //Retorna el total de diners pagats en carburant des de l'inici
        let emplenaments = self.obtenirEmplenaments(false)
        
        var quantitat = 0.0
        for emplenament in emplenaments {
            quantitat += emplenament.costDouble
        }
        return quantitat
    }
    
    func totalMantenimentPagat() -> Double {
        //Retorna el total de diners pagats en despesede manteniment
        var quantitat = 0.0
        
        let registreManteniments = self.registreManteniment as! NSMutableSet
        for manteniment in registreManteniments {
            quantitat += (manteniment as! RegistreManteniment).costDouble
        }
        
        return quantitat
    }
    
    func totalImpostosPagat() -> Double {
        //Retorna el total de diners pagats en imposts el darrer any
        let kDuradaAny = 365.24
        return (self.costDiariImpost() * kDuradaAny) + (self.costDiariSeguro() * kDuradaAny)
    }
    
    func totalPagat() -> Double {
        //Retorna el total pagat d'el vehicle
        //NOTA: per ara els imposts i seguro es calcula només el darrer any.... cal millorar implementació per controlar hitòric d'aquests costs
        let costImpostos = self.totalImpostosPagat()
        let costManteniment = self.totalMantenimentPagat()
        let costCarbutant = self.totalCarburantPagat()
        
        return costImpostos + costManteniment + costCarbutant
    }
    
    func obtenirCarburants() -> [Carburant] {
        //Retorna un array amb la llista dels carburants
        if self.carburants.count > 0 {
            let retornar:[Carburant] = (self.carburants.allObjects as! [Carburant])
            return retornar
        } else {
            return []
        }
    }
    
    func obtenirRegistreManteniments() -> [RegistreManteniment] {
        //Retorna TOTS els manteniments, ordenats per data de més recent a menys
        if self.registreManteniment!.count > 0 {
            let ordre = NSComparisonResult.OrderedDescending
            return ((self.registreManteniment!.allObjects) as! [RegistreManteniment]).sorted({ $0.data.compare($1.data) == ordre})
        } else {
            return []
        }
    }

    func obtenirEmplenaments(ascendent: Bool) -> [Emplenament] {
        //Retorna TOTS els emplenaments, ordenats segons criteri del paràmetre, l'ordre s'aplica segons el contaquilòmetres del vehicle
        //al realitzar l'emplenament, ja que es concidera més rellevant que la data
        if self.emplenaments!.count > 0 {
            var ordre = NSComparisonResult.OrderedDescending
            if ascendent == true {
                ordre = NSComparisonResult.OrderedAscending
            }
            return ((self.emplenaments!.allObjects) as! [Emplenament]).sorted({ $0.km.compare($1.km) == ordre})
        } else {
            return []
        }
    }
    
    func obtenirEmplenaments(ascendent: Bool, quilometres: Double) -> [Emplenament] {
        //Retorna els emplenaments ordenats segons criteri del paràmetre i a partir de una distànca conctreta, l'ordre s'aplica segons el contaquilòmetres del vehicle
        //al realitzar l'emplenament, ja que es concidera més rellevant que la data
        if self.emplenaments!.count > 0 {
            var ordre = NSComparisonResult.OrderedDescending
            if ascendent == true {
                ordre = NSComparisonResult.OrderedAscending
            }
            var retorn = self.emplenaments!.allObjects.sorted({ $0.km.compare($1.km) == ordre}) as! [Emplenament]
            retorn = (retorn.filter(){
                if Double(($0 as Emplenament).km) > quilometres && ascendent {
                    return true
                } else if Double(($0 as Emplenament).km) < quilometres && ascendent == false {
                    return true
                } else  {
                    return false
                }
                }) as [Emplenament]
            return retorn
        } else {
            return []
        }
    }
    
    func obtenirEmplenaments(ascendent: Bool, data: NSDate) -> [Emplenament] {
        //Retorna els emplenaments ordenats segons criteri del paràmetre i a partir de una data conctreta, l'ordre s'aplica segons el contaquilòmetres del vehicle
        //al realitzar l'emplenament, ja que es concidera més rellevant que la data
        if self.emplenaments!.count > 0 {
            var ordre = NSComparisonResult.OrderedDescending
            if ascendent == true {
                ordre = NSComparisonResult.OrderedAscending
            }
            var retorn = (self.emplenaments!.allObjects.filter(){
                if ($0 as! Emplenament).data.compare(data) == NSComparisonResult.OrderedAscending && ascendent {
                    return true
                } else if ($0 as! Emplenament).data.compare(data) == NSComparisonResult.OrderedDescending && ascendent == false {
                    return true
                } else  {
                    return false
                }
                }) as! [Emplenament]
            retorn = retorn.sorted({ $0.km.compare($1.km) == ordre})
            return retorn
        } else {
            return []
        }
    }
    
    func obtenirEmplenaments(ascendent: Bool, max: Int, ple: Bool) -> [Emplenament] {
        //Retorna el número indicat d'emplenaments segons l'ordre indicat.
        //Respecte el número d'emplenaments a tornar, s'hi s'indica que han de ser plens, els emplenaments parcials intermitjos es retornen també
        //però sense contar com a emplenaments, per tant si no es demana que siguin plens i es demanen 3 emplenaments, es retornaran 3 emplenaments (si n'hi ha clar)
        //peró si es demana que 3 emplenaments siguin diposit ple, es retornaran els tres elmenament de diposit ple que hi hagi incloent els pacials fets entre aquests
        if (self.emplenaments!.count ?? 0) > 0 {
            var ordre = NSComparisonResult.OrderedDescending
            if ascendent == true {
                ordre = NSComparisonResult.OrderedAscending
            }
            var totsOrdenats = ((self.emplenaments!.allObjects) as! [Emplenament]).sorted({ $0.km.compare($1.km) == ordre})
            var retorn: [Emplenament] = []
            var index = 0
            var maxRelatiu = max
            while index < totsOrdenats.count && index < maxRelatiu {
                if totsOrdenats[index].pleBool {
                    retorn.append(totsOrdenats[index])
                } else {
                    retorn.append(totsOrdenats[index])
                    maxRelatiu += 1
                }
                index += 1
            }
            return retorn
        } else {
            return []
        }
    }
    
    func obtenirEmplenaments(ascendent: Bool, max: Int, ple: Bool, carburant: String) -> [Emplenament] {
        //Retorna el número indicat d'emplenaments d'un carburant específic segons l'ordre indicat.
        //Respecte el número d'emplenaments a tornar, s'hi s'indica que han de ser plens, els emplenaments parcials intermitjos es retornen també
        //però sense contar com a emplenaments, per tant si no es demana que siguin plens i es demanen 3 emplenaments, es retornaran 3 emplenaments (si n'hi ha clar)
        //peró si es demana que 3 emplenaments siguin diposit ple, es retornaran els tres elmenament de diposit ple que hi hagi incloent els pacials fets entre aquests
        if self.emplenaments!.count > 0 {
            var ordre = NSComparisonResult.OrderedDescending
            if ascendent == true {
                ordre = NSComparisonResult.OrderedAscending
            }
            var filtrats = self.emplenaments!.allObjects.filter({ ($0 as! Emplenament).carburant.nom == carburant })
            var totsOrdenats = ((filtrats) as! [Emplenament]).sorted({ $0.km.compare($1.km) == ordre})
            var retorn: [Emplenament] = []
            var index = 0
            var maxRelatiu = max
            while index < totsOrdenats.count && index < maxRelatiu {
                if totsOrdenats[index].pleBool {
                    retorn.append(totsOrdenats[index])
                } else {
                    retorn.append(totsOrdenats[index])
                    maxRelatiu += 1
                }
                index += 1
            }
            return retorn
        } else {
            return []
        }
    }
    
    func obtenirDarrerEmplenament() -> Emplenament? {
        //Mosrtar l'emplenament més nou
        if (self.emplenaments?.count ?? 0) > 0{
            return self.obtenirEmplenaments(false, max: 1, ple: false).first!
        } else {
            return nil
        }
    }
    
    func obtenirPrimerEmplenament() -> Emplenament? {
        //Mostra l'emplenament més antic
        if self.emplenaments!.count > 0{
            return self.obtenirEmplenaments(true, max: 1, ple: false).first!
        } else {
            return nil
        }
    }
    
    func obtenirDarrerCarburant() -> Carburant? {
        //Saber quin és l'últim carburant útilitzat
        if self.emplenaments!.count > 0{
            let emplenaments: [Emplenament] = self.obtenirEmplenaments(false)
            return emplenaments[0].carburant
        } else {
            return nil
        }
    }
    
    func obtenirKm() -> NSNumber {
        //Saber el contaquilòmetres actual de vehicle
        if self.emplenaments!.count > 0 {
            var emplenaments: [Emplenament] = self.obtenirEmplenaments(true)
            emplenaments.sort(self.oerdenarKmASC)
            return emplenaments[0].km
        } else {
            return (0 as NSNumber)
        }
    }
    
    func oerdenarKmASC(this:Emplenament, that:Emplenament) -> Bool {
        //Funció que permet l'ordenació segons els KM entre dos emplenaments
        return (this.km as! Double) > (that.km as! Double)
    }
    
    private func stringAData(entrada: String) -> NSDate {
        //Transforma l'string passat a data
        let formatter = NSDateFormatter()
        formatter.locale = NSLocale(localeIdentifier: "ES_es")
        //formatter.dateFormat = "E, dd MMM yyyy HH:mm:ss Z"
        formatter.dateFormat = "dd/MM/yyyy"
        //let date = formatter.dateFromString("Thu, 04 Sep 2014 10:50:12 +0000")
        return formatter.dateFromString(entrada)!
    }
    
    private func dataAString (data: NSDate?) -> String {
        //Transforma la data a un string
        let formatter = NSDateFormatter()
        formatter.locale = NSLocale(localeIdentifier: "ES_es")
        //formatter.dateFormat = "E, dd MMM yyyy HH:mm:ss Z"
        formatter.dateFormat = "dd/MM/yyyy"
        if data != nil {
            return formatter.stringFromDate(data!)
        }else {
            return ""
        }
    }
    
    func guardarDataCompra(entrada: String) {
        //Guarda la data de compra
        if entrada != "" {
            self.dataCompra = self.stringAData(entrada)
        }
    }
    
    func guardaDataAdquisicio(entrada: String){
        //Guarda la data d'adquisició
        if entrada != "" {
            self.dataAdquisicio = self.stringAData(entrada)
        }
    }
    
    func obtenirDataCompra() -> String {
        //LLegeix la data de compra i la retorna en format string
        if self.dataCompra != nil {
            return self.dataAString(self.dataCompra)
        } else {
            return ""
        }
    }
    
    func obtenirDataAdquisicio() -> String {
        //Llegeix la data d'adquisició i la retorna en forma string
        if self.dataAdquisicio != nil {
            return self.dataAString(self.dataAdquisicio)
        } else {
            return ""
        }
    }
    
    func obtenirConsums(maxim: Int) -> ([Double],[NSDate]) {
        //Retorna els últims darrers consums calculables i la d'ata d'aquests, el número de consums retornat és indicat per el maxim.
        let emplenaments = self.obtenirEmplenaments(false)
        
        var maximRelatiu = maxim
        var parcial = false
        
        var distancia: Double = 0
        var litres: Double = 0
        
        var retorn: [Double] = []
        var dates: [NSDate] = []
        for var index = 0; (index < maximRelatiu && (index < emplenaments.count - 1)); ++index {
            let emplenamentAnterior = emplenaments[index + 1]
            let emplenament = emplenaments[index]
            
            distancia += Double(emplenament.km) - Double(emplenamentAnterior.km)
            litres += Double(emplenament.litres)
            
            if Bool(emplenament.ple) {
                parcial = false
                retorn.append(Double((litres/distancia))*100)
                litres = 0
                distancia = 0
                dates.append(emplenament.data)
            }   else {
                parcial = true
                maximRelatiu += 1 //Augmentem el maxim, ja que un parcial no dona un resultat
            }
        }
        return (retorn,dates)
        
        //Un altre sistema d'obtenir-los que semblava que havia de ser més eficient....
        /*
        var retorn: [Double] = []
        var dates: [NSDate] = []
        
        if var emplenament = self.obtenirDarrerEmplenament() {
            var maximRelatiu: Int = maxim
            var distancia: Double = 0
            var litres: Double = 0
            
            for var index = 0; index < maximRelatiu && emplenament.anterior != nil; ++index {
                distancia += emplenament.contaquilometres - emplenament.anterior!.contaquilometres
                litres += emplenament.quantitat
                
                if emplenament.pleBool {
                    
                    retorn.append(Double((litres/distancia))*100)
                    dates.append(emplenament.data)
                    
                    litres = 0
                    distancia = 0
                } else {
                    maximRelatiu += 1
                }
                emplenament = emplenament.anterior!
            }
        }

        return (retorn,dates)*/
    }
    
    func obtenirCostDiari() -> Double {
        //Retorna el cost fixe diari total del vehicle, o sigui, el que costa el vehicle encara que no es faci servir
        return (self.costDiariSeguro() + self.costDiariImpost())
    }
    
    func costDiariSeguro() -> Double {
        //Calcula segons els paràmetres del seguro, el cost diari d'aquest
        let kDuradaAny = 365.24
        if self.seguro != nil {
            switch self.seguro!.pagament as String {
            case "Anual":
                return (self.seguro!.preu as! Double) / kDuradaAny
            case "Bimestral":
                return (self.seguro!.preu as! Double) / (kDuradaAny / 2)
            case "Trimestral":
                return (self.seguro!.preu as! Double) / (kDuradaAny / 3)
            default: break;
            }
        }
        return 0.0
    }
    
    func costDiariImpost() -> Double {
        //Calcula segons els paràmetres de l'impost el cost diari d'aquest
        let kDuradaAny = 365.24
        if self.impostMunicipal != nil {
            switch self.impostMunicipal!.pagament as String {
            case "Anual":
                return (self.impostMunicipal!.preu as! Double) / kDuradaAny
            case "Bimestral":
                return (self.impostMunicipal!.preu as! Double) / (kDuradaAny / 2)
            case "Trimestral":
                return (self.impostMunicipal!.preu as! Double) / (kDuradaAny / 3)
            default: break;
            }
        }
        return 0.0
    }
    
    func obtenirCostCarburantPerUnitatDistancia() -> Double? {
        //Calcula quant diners per quilòmetre consumeix el vehicle en carburant
        var consumMig: Double? = nil
        if let carburantActual = self.obtenirDarrerCarburant() {
            let emplenaments = self.obtenirEmplenaments(false, max: 3, ple: true, carburant: carburantActual.nom)
            var count = 0
            if emplenaments.count > 1 {
                for var i = 0; i<emplenaments.count; ++i {
                    if consumMig == nil &&  emplenaments[i].costDistancia != nil{
                        consumMig = emplenaments[i].costDistancia
                        count += 1
                    } else if emplenaments[i].costDistancia != nil {
                        consumMig = consumMig! + emplenaments[i].costDistancia!
                        count += 1
                    }
                }
                
                //Fem la mitjana de cost
                if consumMig != nil {
                    consumMig = consumMig! / Double(count)
                }
            }
        }
        return consumMig
    }
    
    func obtenirCostPerUnitatDistancia() -> Double? {
        //Calcula quants diners per quilòmetre en total consumeix el vehicle (carburant + desgast)
        var consumMig: Double? = self.obtenirCostCarburantPerUnitatDistancia()
        
        //Ara cal incloir el desgast de les peces de manteniment
        if let costUnitatDistancia = self.obtenirCostDesgastPerUnitatDistancia() {
            if consumMig == nil {
                consumMig = costUnitatDistancia
            } else {
                consumMig = consumMig! + costUnitatDistancia
            }
        }
    
        return consumMig
    }
    
    func obtenirCostDesgastPerUnitatDistancia() -> Double? {
        //calcula quants diners relatius endesgast de peca es generen per a cada quilòmetre recorregut
        var consumMig: Double? = nil
        
        for tipusManteniment in Manteniments().kLlistaTipus {
            //Per a cada tipus de manteniment, n'agafarem el darrer registre i en calcularem el cost
            let manteniments = self.obtenirRegistresDeMantenimentDe(tipusManteniment)
            if manteniments.count > 0 {
                if manteniments[0].distanciaMaxima != nil {
                    //calculem el preu per unitat distància
                    let preuPerUnitatDistancia = Double(manteniments[0].cost) / Double(manteniments[0].distanciaMaxima!)
                    if consumMig == nil {
                        consumMig = preuPerUnitatDistancia
                    } else {
                        consumMig = consumMig! + preuPerUnitatDistancia
                    }
                }
            }
        }
        
        return consumMig
    }
    
    func obtenirRegistresDeMantenimentDe(nom: String) -> [RegistreManteniment] {
        //Obtenimt d'un manteniment concret, tots els registres ordenats de més recent a meny.
        var retorn: [RegistreManteniment] = []
        
        let manteniments = self.registreManteniment?.allObjects.filter({ ($0 as! RegistreManteniment).nom == nom })
        if manteniments?.count > 0 {
            //Tenim manteniments del tipus indicat, els ordenarem per data
            let ordre = NSComparisonResult.OrderedDescending
            retorn = (manteniments as! [RegistreManteniment]).sorted(({ $0.data.compare($1.data) == ordre}))
        }
        
        return retorn
    }

}
